//=========================================================
//+ rcx6.c - main program for rover remote control at RCX end
// 
// Single Author info:
// 
// username: Xuejun Sun
// Group 4
//
// Group info:
//
// username: Haigui Huang
//
//==========================================================
//
#include <lnp.h>
#include <conio.h>
#include <sys/irq.h>
#include <sys/h8.h>
#include <tm.h>
#include <dkey.h>
#include <unistd.h>
#include <lnp-logical.h>
#include <sys/lnp-logical.h>
#include <dsound.h>
#include <stdlib.h>
#include "remote_common.h"
#include "switch_case_motor_control.h"

#define MY_PORT 2				// my port
#define MY_PORT_1  7				// host port

#define DEST_HOST 0x08				// host seq
#define DEST_PORT 0x07				// host port
#define DEST_ADDR ( DEST_HOST << 4 | DEST_PORT )// 0x87, host address

static int done;

unsigned char lnpMsg[10] = {'a'};	// msg buffer

// last LNP messages broadcasted from every RCX
char LastLnpMsg[10];
int  lnpMsgRecvd = 0;   // LNP message received for this RCX
int  lnpMsgLen = 0;
int  lnpPos = 0;

// Our LNP message handler
void  my_lnp_message_handler(const unsigned char *data, unsigned char length)
{
	lnpMsgLen = length;
        //bytes = length-1;
        memcpy( LastLnpMsg, data, length);
        //for(; bytes >=0; bytes-- );
        //        LastLnpMsg[bytes] = data[bytes];
        //LastLnpMsg[length] = '\0';
	
        lnpMsgRecvd = 1;        // set flag to TRUE
        //dsound_system(0);
        //cputs(data);
}

// main function for
// 	retrieving command message from buffer
// 	control motor according to the command from the message
// 	write the currently changed rover status if there is a change
int main(int argc, char *argv[])
{
  // input data, including inCode and inStatus
  int inCode = 5;			// comand code, initially stop
  int inStatus = 0;			// input status flag

  int preInCode = 99;			// previous command code

  char buf[5];

  stateSpeed = 's';			// initiallized as stop state

  turnRate = 1; 	// initialize one bit shift for turn ratio
  turnPressNum = 0;	// ... turn bit number

  curSpeed = 0 ; 			// initialized at 0 speed

  // stop the motor
  rcx_stop();

  // set up lnp short/long comm. range, handler and rcx lcd
  lnp_logical_range(1);
  lcd_clear();
  lnp_addressing_set_handler(MY_PORT, my_lnp_message_handler);

  inStatus = 1;

  lnpMsg[1] = '\0';

  // main loop for receive command, conduct motor control, and send back msg
  while ( 1 )
  {
	// command message prcessing
      if ( lnpMsgRecvd > 0)
      {
        memcpy(buf, LastLnpMsg, lnpMsgLen);
        inCode = buf[0] ;
        lnpMsgRecvd = 0;
        buf[0] = inCode + '0';
        buf[1] = '\0';
        cputs(buf);
        inStatus = 1;
      }
      else   // no LNP message received
      {
          continue;
      }

      // init the other part of char string for lcd display
      buf[2] = ' ';
      buf[3] = inCode + '0';
      buf[4] = '\0';
      cputs(buf);

     // rover controled with inCode
     if ( (inCode >= 1 ) && ( inCode <= 5 ) && ( inStatus == 1 ) )
     {
         // count if turning press contineously
         if ( ( inCode != 3 ) && ( inCode != 4 ) ) {
             turnPressNum = 0;
         }
         else if  ( inCode == 3 ) {
             if ( preInCode == inCode )
             turnPressNum++;
             else
             turnPressNum = 1;
         }
         else if  ( inCode == 4 ) {
             if ( preInCode == inCode )
             turnPressNum++;
             else
             turnPressNum = 1;
         }
         dsound_system(0);

	 // motor control
         switch_case_motor_control(inCode);
         preInCode = inCode;

         buf[0] = '0' + ( comMsg >> 6 );
         buf[1] = '0' + ((comMsg << 2 ) >> 3);
         cputs(buf);

         // send message out	
         lnpMsg[0] = comMsg;
         lnp_addressing_write(lnpMsg,1,DEST_ADDR, MY_PORT );

         inStatus = 0;
     }//     if ( (inCode >= 1 ) && ( inCode <= 8 ) && ( inStatus == 1 ) ) {

    }// end of while(1)

    lnp_addressing_set_handler(MY_PORT,LNP_DUMMY_ADDRESSING);
    return 0;
}

