//=========================================================
//+ switch_case_motor_control.h - head for motor control module
//
// Single Author info:
// 
// username: Xuejun Sun
// Group 4
// 
// Group info:
// 
// username: Haigui Huang
//
//==========================================================
// 
// switch case motor control
// up-stream: command from PC
// code:
// 1 - speedup
// 2 - slowdon
// 3 - turn left
// 4 - turn right
// 5 - stop
// ---------------------------------
// 5 - speed step increase
// 6 - speed step decrease
// 7 - direction degree step increase
// 8 - direction degree step decrease
//
// status: 0 - old
//         1 - new
//
// down-stream: measured value message from RCX
// code: speed/dir
// status: 

#include <unistd.h>
#include <conio.h>
#include <dmotor.h>
#include <dsound.h>
#include "remote_common.h"

// stop motor
static inline void rcx_stop()
{
    stateSpeed = 's';
    curSpeed = 0;
    motor_c_speed(0);
    motor_a_speed(0);
    motor_a_dir(fwd);
    motor_c_dir(fwd);
}

// go forward
static inline void go_forward() {
    motor_a_dir(fwd);
    motor_c_dir(fwd);

    motor_c_speed(curSpeed);
    motor_a_speed(curSpeed);
}

// go backward
static inline void go_backward() {
    motor_a_dir(rev);
    motor_c_dir(rev);

    motor_c_speed(curSpeed);
    motor_a_speed(curSpeed);
}

// speedup
// note: the control may be contineously from going backword to forward.
//       a finite state machine required to let it passover the stop phase
static inline void button_up() {
  // stateSpeed need to be globle. f - forward, b - backward, s - stop
  if ( (stateSpeed == 'f') || (stateSpeed == 's') )
  {						// currently s or f state
      if  ( curSpeed > ( MAX_SPEED - speedStep ) ) { // upper speed bounary
          curSpeed = MAX_SPEED;
      }
      else    {
          curSpeed += speedStep;
      }
      stateSpeed = 'f';
      go_forward(curSpeed);	
  }
  else if ( stateSpeed == 'b' )
  {						// currently backward state
      if ( curSpeed < speedStep ) {		// from backward to 0 speed
          curSpeed = 0;
	  stateSpeed = 's';
          rcx_stop();
      }
      else {					
         curSpeed -= speedStep;
         go_backward();
     }
  }
}

// speeddown
// note: the control may be contineously from going forward to backward.
//       a finite state machine required to let it passover the stop phase
static inline void button_down() {
  // stateSpeed need to be globle. f - forward, b - backward
  if ( (stateSpeed == 'b') || (stateSpeed == 's' ) ){
	  				// currently at 'b' or 's' state
    if( curSpeed > MAX_SPEED - speedStep )
	    curSpeed = MAX_SPEED;
    else
	    curSpeed += speedStep;
    stateSpeed = 'b';
    go_backward();
  }
  else {				// currently at forward state
    if (curSpeed < speedStep ) {
	    curSpeed = 0 ;
	    stateSpeed = 's';
	    rcx_stop();
    }
    else {
      curSpeed = curSpeed - speedStep ;
      go_forward();
    }
  }
}

// highest bit of curSpeed
static inline int hNormalSpeedBit() { // counted from high to low
  int i;
	for (i = 0; i < 8; i++) {
	  if ( ( ( bit2 >> i ) | curSpeed ) != 0 ) {
	    return i;
	  }
	}
	return 0;
}

// speed code for inner wheel in turning
static inline int inWheelSpeed() {
  int tmp =  hNormalSpeedBit();
  if ( tmp > 4 ) 
    return curSpeed >> 1;
  tmp = curSpeed >> 3;  // delta speed - 1/8 normal
  if ( turnPressNum > 7 ) turnPressNum = 7;
  return ( curSpeed - turnPressNum * tmp) ;
}

// speed code for outer wheel in turning
static inline int outWheelSpeed() {
  int tmp =  hNormalSpeedBit();
  if ( tmp < 1 ) 
    return curSpeed;
  if ( tmp > 4 )
    tmp = speedStep >> 1;          // delta speed
  else
    tmp = curSpeed >> 3;  // delta speed - 1/8 normal
  if ( turnPressNum > 7 ) turnPressNum = 7;
  tmp = turnPressNum * tmp;
  if ( ( MAX_SPEED - tmp ) <= curSpeed )
    return MAX_SPEED;
  return ( curSpeed + tmp) ;
}

// rover left turn
static inline void turn_left() {
    motor_a_speed( inWheelSpeed() );
    motor_c_speed( outWheelSpeed() );
    if (stateSpeed == 'f') {
    	motor_a_dir(fwd);
    	motor_c_dir(fwd);
    }
    else {
	motor_a_dir(rev);
	motor_c_dir(rev);
    }
}

// right turn
static inline void turn_right() {
    motor_c_speed( inWheelSpeed() );
    motor_a_speed( outWheelSpeed() );
    if (stateSpeed == 'f') {
    	motor_a_dir(fwd);
    	motor_c_dir(fwd);
    }
    else {
       motor_a_dir(rev);
       motor_c_dir(rev);
    }

}

// motor control logic / FSM
static inline void switch_case_motor_control(int incode) {

  //  unsigned char msgSpeedTurn = 0;
  //  unsigned char accumTurn = 0;

  switch (incode) {

  case 1:				// button up (speed)
    button_up();
    if ( stateSpeed == 'f' ) 
      comMsg = bit0 | ( curSpeed >> 3 ) ;
    else if ( stateSpeed == 'b' )
      comMsg = bit1 | ( curSpeed >> 3 ) ;
    else if ( stateSpeed == 's' )
      comMsg = bit0 | ( curSpeed >> 3 ) ;
    break;

  case 2:				// button down
    button_down();
    if ( stateSpeed == 'f' )
      comMsg = bit0 |( curSpeed >> 3 )  ;
    else if ( stateSpeed == 'b' )
      comMsg = bit1 |( curSpeed >> 3 ) ;
    else if ( stateSpeed == 's' )
      comMsg = bit0 | ( curSpeed >> 3 ) ;
    break;

  case 3:				// button left
    turn_left();
    comMsg = bit2 | turnPressNum; // accumTurn to be implemented
    break;

  case 4:				// button right
    turn_right();
    comMsg = bit3 | turnPressNum;
    break;

  case 5:				// stop
    rcx_stop();
    comMsg = bit0;
    break;

  default:				// otherwise stop
     rcx_stop();
     break;
  }
}
